/*******************************************************************************
********************************************************************************

   COPYRIGHT  2004, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   Compiler.h     Common compiler definitions for the S1000 Strobe Controller.

   About:         This module contains common compiler definitions
                  of the S1000 Strobe Controller in the AICONS01 project.

   Target:        Win32 Compatible

   Written for:   Advanced Illumination, Inc.
                  24 Peavine Drive
                  Rochester, VT 05767
                  http://advancedillumination.com

   Contacts:      John Thrailkill
                  Phone:  (802) 767-3830
                  FAX:    (802) 767-3831
                  jthrailkill@advill.com

   Written by:    Rob Macklin
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 4 $

--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/S1000/pc_src/Common/compiler.h $
 * 
 * 4     9/23/04 9:34a Rmack
 * Added SWAP_FLOAT32_M
 * 
 * 3     9/07/04 12:45p Rmack
 * Added bit definitions.
 * 
 * 2     8/24/04 8:44a Rmack
 * Updated header information.
 * 
 * 1     3/30/04 3:27p Rmack
 * 
********************************************************************************
*******************************************************************************/
#ifndef _COMPILER_H
#define _COMPILER_H


/* MPD Standard data type definitions */
#ifndef MPD_TYPES
#define MPD_TYPES
typedef unsigned char  UINT8_T;
typedef unsigned short UINT16_T;
typedef unsigned long  UINT32_T;
typedef char           INT8_T;
typedef short          INT16_T;
typedef long           INT32_T;
typedef unsigned char  BOOLEAN_T;
typedef float          FLOAT32_T;
typedef INT16_T        ERRCODE_T;
typedef INT8_T (*FUNC_PTR_T)(void); 
#endif

#ifndef E_SUCCESS
#define E_SUCCESS                         0
#endif
                     
/* The following define can be used to force   */
/* global visibility of local functions        */
#if (DEBUG_ENABLE == 0)
#define STATIC static 
#else
#define STATIC
#endif

/*
	Definitions
*/

#define LO_BYTE_MASK		0x00FF	  /* Mask out upper byte keep low  */
#define HI_BYTE_MASK		0xFF00	  /* Mask out lower byte keep high */  
#define LO_WORD_MASK    0x0000FFFF /* Mask out upper word keep low */
#define HI_WORD_MASK    0xFFFF0000 /* Mask out lower word keep high */
#define TRUE				1
#define FALSE				0

#ifdef _WIN32
#include <windows.h>
#define HI_BYTE(a)      HIBYTE(a)
#define LO_BYTE(a)      LOBYTE(a)
#define HI_WORD(a)      HIWORD(a)
#define LO_WORD(a)      LOWORD(a)
#else   
#define HI_BYTE(a)      ((UINT16_T) (a) >> 8)
#define LO_BYTE(a)      ((a) & 0xFF)
#define HI_WORD(a)      ((UINT32_T) (a) >> 16)
#define LO_WORD(a)      ((a) & 0xFFFF)
#endif

#define NOT             ! 
#define IS_EVEN(a)      (((a) & 0x1) == 0)
#define IS_ODD(a)       (NOT IS_EVEN(a))
#define ASCII_CASE_BIT  0x20
#define IS_UPPERCASE(a)   (((a) & ASCII_CASE_BIT) == 0)
#define IS_LOWERCASE(a)   (((a) & ASCII_CASE_BIT) != 0)
#define MAKE_UPPERCASE(a) ((UINT8_T) ((a) & ~ASCII_CASE_BIT))
#define MAKE_LOWERCASE(a) ((UINT8_T) ((a) | ASCII_CASE_BIT))
#define LENGTH(a)         (sizeof(a)/sizeof(a[0]))

/* Macros to handle byte swapping for the SZ328 */
#define SWAP_UINT16_M(w)   MAKEWORD(HIBYTE(w),LOBYTE(w))
#define SWAP_UINT32_M(l)   MAKELONG(                     \
                              MAKEWORD(                  \
                                 HIBYTE(HIWORD(l)),      \
                                 LOBYTE(HIWORD(l))       \
                              ),                         \
                              MAKEWORD(                  \
                                 HIBYTE(LOWORD(l)),      \
                                 LOBYTE(LOWORD(l))       \
                              )                          \
                           )



#define SWAP_FLOAT32_M(val)      swap_float32(val)

static FLOAT32_T swap_float32(FLOAT32_T val)
{
   union
   {
      FLOAT32_T f;
      UINT32_T  u;
   } swap_val;

   swap_val.f = val;

   swap_val.u = SWAP_UINT32_M(swap_val.u);

   return(swap_val.f);
}



#define BIT0                0x00000001
#define BIT1                0x00000002
#define BIT2                0x00000004
#define BIT3                0x00000008
#define BIT4                0x00000010
#define BIT5                0x00000020
#define BIT6                0x00000040
#define BIT7                0x00000080
#define BIT8                0x00000100
#define BIT9                0x00000200
#define BIT10               0x00000400
#define BIT11               0x00000800
#define BIT12               0x00001000
#define BIT13               0x00002000
#define BIT14               0x00004000
#define BIT15               0x00008000
#define BIT16               0x00010000
#define BIT17               0x00020000
#define BIT18               0x00040000
#define BIT19               0x00080000
#define BIT20               0x00100000
#define BIT21               0x00200000
#define BIT22               0x00400000
#define BIT23               0x00800000
#define BIT24               0x01000000
#define BIT25               0x02000000
#define BIT26               0x04000000
#define BIT27               0x08000000
#define BIT28               0x10000000
#define BIT29               0x20000000
#define BIT30               0x40000000
#define BIT31               0x80000000


#endif /* COMPILER_H */