/*******************************************************************************
********************************************************************************

   COPYRIGHT  2004, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   usbdefs.h      USB Descriptor Header File 

   About:         This is the common header file for the USB Descriptor
                  information for the AICONS01 project.

   Target CPU:    H8S/2215

   Written for:   Advanced Illumination, Inc.
                  24 Peavine Drive
                  Rochester, VT 05767
                  http://advancedillumination.com

   Contacts:      John Thrailkill
                  Phone:  (802) 767-3830
                  FAX:    (802) 767-3831
                  jthrailkill@advill.com

   Written by:    Rob Macklin
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 1 $

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/Pulsar320/tar_src/H8App/usbdefs.h $
 * 
 * 1     8/15/08 8:45a Jpiet
 * Initial check-in
 * 
 * 1     3/30/04 3:32p Rmack
 *
********************************************************************************
*******************************************************************************/
#ifndef _USBDEFS_H
#define _USBDEFS_H

#include "hostcmds.h"
/*******************************************************************************
 *******************************************************************************
								  Definitions
********************************************************************************
*******************************************************************************/

// USB Status Codes
//
#define USBDEFS_STATUS_ATTACHED           0x00
#define USBDEFS_STATUS_POWERED            0x01
#define USBDEFS_STATUS_DEFAULT            0x02
#define USBDEFS_STATUS_ADDRESSED          0x03
#define USBDEFS_STATUS_CONFIGURED         0x04
#define USBDEFS_STATUS_SUSPENDED          0x80

// USB Setup Request Types
//
#define USBDEFS_RQSTTYPE_STD              0x00
#define USBDEFS_RQSTTYPE_CLASS            0x20
#define USBDEFS_RQSTTYPE_VENDOR           0x40
#define USBDEFS_RQSTTYPE_MASK             (USBDEFS_RQSTTYPE_CLASS | USBDEFS_RQSTTYPE_VENDOR)


//----------------------------------------------------------------------------

// USB Standard Device Request Codes
// according to USB1.1 spec page 187
//
#define USBDEFS_DEVRQST_GET_STATUS        0x00
#define USBDEFS_DEVRQST_CLEAR_FEATURE     0x01
#define USBDEFS_DEVRQST_SET_FEATURE       0x03
#define USBDEFS_DEVRQST_SET_ADDRESS       0x05
#define USBDEFS_DEVRQST_GET_DESCRIPTOR    0x06
#define USBDEFS_DEVRQST_SET_DESCRIPTOR    0x07
#define USBDEFS_DEVRQST_GET_CONFIGURATION 0x08
#define USBDEFS_DEVRQST_SET_CONFIGURATION 0x09
#define USBDEFS_DEVRQST_GET_INTERFACE     0x0a
#define USBDEFS_DEVRQST_SET_INTERFACE     0x0b
#define USBDEFS_DEVRQST_SYNCH_FRAME       0x0c

//----------------------------------------------------------------------------

// USB Class-Specific Device Request Codes
// according to USB1.1
//
#define USBDEFS_DEVCLASSRQST_GET_REPORT   0x01
#define USBDEFS_DEVCLASSRQST_GET_IDLE     0x02
#define USBDEFS_DEVCLASSRQST_GET_PROTOCOL 0x03
#define USBDEFS_DEVCLASSRQST_SET_REPORT   0x09
#define USBDEFS_DEVCLASSRQST_SET_IDLE     0x0A
#define USBDEFS_DEVCLASSRQST_SET_PROTOCOL 0x0B

//----------------------------------------------------------------------------

// Descriptor Types
// according to USB1.1 spec page 187
//
#define USBDEFS_DEVICE_DESCRIPTOR_TYPE    0x01
#define USBDEFS_CONFIG_DESCRIPTOR_TYPE    0x02
#define USBDEFS_STRING_DESCRIPTOR_TYPE    0x03
#define USBDEFS_INTERFACE_DESCRIPTOR_TYPE 0x04
#define USBDEFS_ENDPOINT_DESCRIPTOR_TYPE  0x05
#define USBDEFS_POWER_DESCRIPTOR_TYPE     0x06
#define USBDEFS_HID_DESCRIPTOR_TYPE       0x21
#define USBDEFS_REPORT_DESCRIPTOR_TYPE    0x22
#define USBDEFS_PHYSICAL_DESCRIPTOR_TYPE  0x23

#define USBDEFS_MAXIMUM_USBDEFS_STRING_LENGTH	255

// values for the bits returned by the USB GET_STATUS command
#define USBDEFS_GETSTATUS_SELF_POWERED          0x01
#define USBDEFS_GETSTATUS_REMOTE_WAKEUP_ENABLED 0x02

//
// Values for bmAttributes field of an
// endpoint descriptor
//
#define USBDEFS_ENDPOINT_TYPE_MASK          0x03

#define USBDEFS_ENDPOINT_TYPE_CONTROL       0x00
#define USBDEFS_ENDPOINT_TYPE_ISOCHRONOUS   0x01
#define USBDEFS_ENDPOINT_TYPE_BULK          0x02
#define USBDEFS_ENDPOINT_TYPE_INTERRUPT     0x03

//
// definitions for bits in the bmAttributes field of a 
// configuration descriptor.
//
#define USBDEFS_CONFIG_POWERED_MASK         0xC0

#define USBDEFS_CONFIG_BUS_POWERED          0x80
#define USBDEFS_CONFIG_SELF_POWERED         0xC0
#define USBDEFS_CONFIG_REMOTE_WAKEUP        0x20

//
// Endpoint direction bit, stored in address
//

#define USBDEFS_ENDPOINT_DIRECTION_MASK     0x80

//
// defined USB device classes
//

#define USBDEFS_DEVICE_CLASS_RESERVED           0x00
#define USBDEFS_DEVICE_CLASS_AUDIO              0x01
#define USBDEFS_DEVICE_CLASS_COMMUNICATIONS     0x02
#define USBDEFS_DEVICE_CLASS_HUMAN_INTERFACE    0x03
#define USBDEFS_DEVICE_CLASS_MONITOR            0x04
#define USBDEFS_DEVICE_CLASS_PHYSICAL_INTERFACE 0x05
#define USBDEFS_DEVICE_CLASS_POWER              0x06
#define USBDEFS_DEVICE_CLASS_PRINTER            0x07
#define USBDEFS_DEVICE_CLASS_STORAGE            0x08
#define USBDEFS_DEVICE_CLASS_HUB                0x09
#define USBDEFS_DEVICE_CLASS_VENDOR_SPECIFIC    0xFF


/*	
	General USB Definitions
*/
/* 
   This number should be the total number of endpoints including
   the Control Endpoint.  Buffers are allocated for this many
   endpoints.  Make sure the structure usbdefs_desc_struct has
   an entry for all non-Control endpoints defined by this number.

   Currently the H8 can handle 6 total endpoints, but this code
   is setup for only 5, the additional endpoints are defined
   but not used.
*/
#define USBDEFS_NUM_ENDPOINTS                5
#define USBDEFS_RX_BUFFER_SIZE               140


#define USBDEFS_ENDPOINT_CONTROL             0
#define USBDEFS_ENDPOINT_1                   1
#define USBDEFS_ENDPOINT_2                   2
#define USBDEFS_ENDPOINT_3                   3
#define USBDEFS_ENDPOINT_4                   4
#define USBDEFS_ENDPOINT_5                   5

/* Definitions for endpoint transfer modes */
#define USBDEFS_EP_MODE_DISABLE              -1
#define USBDEFS_EP_MODE_CTRL                 0
#define USBDEFS_EP_MODE_ISOC                 1
#define	USBDEFS_EP_MODE_BULK                2
#define	USBDEFS_EP_MODE_INTR                3

/* Definitions for Device Config change flags */
#define USBDEFS_CONFIGURATION_CHG            1
#define USBDEFS_INTERFACE_CHG                2
#define USBDEFS_ADDRESS_CHG                  4

/* Definitions for USB Bus states */
#define USBDEFS_BUS_SUSPEND                  0x10
#define USBDEFS_BUS_RESUME                   0x20
#define USBDEFS_BUS_RESET                    0x40

/* Callback function flags */
#define USBDEFS_ASOF_FLAG				0x01
#define USBDEFS_REMOTE_WAKEUP_EN		0x02
#define USBDEFS_EP_HALT_FLAG			0x03

/* HID Report specifics */
#define USBDEFS_HID_INPUT_REPORT_TYPE        1
#define USBDEFS_HID_OUTPUT_REPORT_TYPE       2
#define USBDEFS_HID_FEATURE_REPORT_TYPE      3


#define USBDEFS_HID_SHORT_REPORT_LEN         20
#define USBDEFS_HID_LONG_REPORT_LEN          120
#define USBDEFS_HID_SHORT_INPUT_REPORT_ID    1
#define USBDEFS_HID_SHORT_INPUT_LEN          USBDEFS_HID_SHORT_REPORT_LEN
#define USBDEFS_HID_SHORT_INPUT_USAGE        HOSTCMD_SHORT_RSP_USAGE

#define USBDEFS_HID_SHORT_OUTPUT_REPORT_ID   USBDEFS_HID_SHORT_INPUT_REPORT_ID
#define USBDEFS_HID_SHORT_OUTPUT_LEN         USBDEFS_HID_SHORT_REPORT_LEN
#define USBDEFS_HID_SHORT_OUTPUT_USAGE       HOSTCMD_SHORT_CMD_USAGE

#define USBDEFS_HID_LONG_INPUT_REPORT_ID     2
#define USBDEFS_HID_LONG_INPUT_LEN           USBDEFS_HID_LONG_REPORT_LEN
#define USBDEFS_HID_LONG_INPUT_USAGE         HOSTCMD_LONG_RSP_USAGE

#define USBDEFS_HID_LONG_OUTPUT_REPORT_ID    USBDEFS_HID_LONG_INPUT_REPORT_ID
#define USBDEFS_HID_LONG_OUTPUT_LEN          USBDEFS_HID_LONG_REPORT_LEN
#define USBDEFS_HID_LONG_OUTPUT_USAGE        HOSTCMD_LONG_CMD_USAGE

#define USBDEFS_HID_FEATURE_REPORT_ID        3
#define USBDEFS_HID_FEATURE_LEN              2
#define USBDEFS_HID_FEATURE_USAGE            HOSTCMD_FEATURE_USAGE


/* String descriptors */
#define USBDEFS_SD1LEN sizeof("Advanced Illumination")*2
#define USBDEFS_SD2LEN sizeof("Ai Pulsar 320 Controller")*2
#define USBDEFS_SD3LEN sizeof("0000000000")*2
#define USBDEFS_SD4LEN sizeof("8200-000000-000")*2
#define USBDEFS_SD5LEN sizeof("7200-000000-000")*2
#define USBDEFS_SD6LEN sizeof("String 6")*2

              
/* HID String Indexes */
#define USBDEFS_HID_MANUFACTURER_STRING      HOSTCMD_MANUFACTURER_STRING
#define USBDEFS_HID_PRODUCT_STRING           HOSTCMD_PRODUCT_STRING
#define USBDEFS_HID_SERIAL_NUMBER_STRING     HOSTCMD_SERIAL_NUMBER_STRING
#define USBDEFS_HID_FW_PARTNUMBER_STRING     HOSTCMD_FW_PARTNUMBER_STRING
#define USBDEFS_HID_HW_PARTNUMBER_STRING     HOSTCMD_HW_PARTNUMBER_STRING



/*******************************************************************************
 *******************************************************************************
								  TypeDefs
********************************************************************************
*******************************************************************************/

// Handle byte ordering in FIFO
typedef struct
{				
	UINT8_T lo;					// (High/Low Byte swapped)
	UINT8_T hi;
} IWORD_T;

// Structure of Setup Packet sent during
// SETUP Stage of Standard Device Requests
// according to USB1.1 spec page 183
//
typedef struct
{
	UINT8_T bmRequestType;           // Characteristics (Direction,Type,Recipient)
	UINT8_T bRequest;                // Standard Request Code
	IWORD_T wValue;                  // Value Field
	IWORD_T wIndex;                  // Index or Offset Field
	IWORD_T wLength;                 // Number of Bytes to transfer (Data Stage)
} USBDEFS_SETUP_BUFFER_T;



// Standard Device Descriptor
// according to USB1.1 spec page 197
//
typedef struct
{
	UINT8_T bLength;                 // Size of this Descriptor in Bytes
	UINT8_T bDescriptorType;         // Descriptor Type (=1)
	IWORD_T bcdUSB;                  // USB Spec Release Number in BCD
	UINT8_T bDeviceClass;            // Device Class Code
	UINT8_T bDeviceSubClass;         // Device Subclass Code	
	UINT8_T bDeviceProtocol;         // Device Protocol Code
	UINT8_T bMaxPacketSize0;         // Maximum Packet Size for EP0 
	IWORD_T idVendor;                // Vendor ID 
	IWORD_T idProduct;               // Product ID
	IWORD_T bcdDevice;               // Device Release Number in BCD
	UINT8_T iManufacturer;           // Index of String Desc for Manufacturer
	UINT8_T iProduct;                // Index of String Desc for Product
	UINT8_T iSerialNumber;           // Index of String Desc for SerNo
	UINT8_T bNumConfigurations;      // Number of possible Configurations
} USBDEFS_DEVICE_DESCRIPTOR_T;

#define USBDEFS_DEVICE_DESC_SIZE    sizeof(USBDEFS_DEVICE_DESCRIPTOR_T)
//#define USBDEFS_DEVICE_DESC_SIZE	18

//----------------------------------------------------------------------------

// Standard Configuration Descriptor
// according to USB1.1 spec page 199
//
typedef struct
{
	UINT8_T bLength;                 // Size of this Descriptor in Bytes
	UINT8_T bDescriptorType;         // Descriptor Type (=2)
	IWORD_T wTotalLength;            // Total Length of Data for this Conf
	UINT8_T bNumInterfaces;          // No of Interfaces supported by this Conf
	UINT8_T bConfigurationValue;     // Designator Value for *this* Configuration
	UINT8_T iConfiguration;          // Index of String Desc for this Conf
	UINT8_T bmAttributes;            // Configuration Characteristics (see below)
	UINT8_T bMaxPower;               // Max. Power Consumption in this Conf (*2mA)
} USBDEFS_CONFIGURATION_DESCRIPTOR_T;

#define USBDEFS_CONFIGURATION_DESC_SIZE	9

//----------------------------------------------------------------------------

// Standard Interface Descriptor
// according to USB1.1 spec page 202
//
typedef struct
{
	UINT8_T bLength;                 // Size of this Descriptor in Bytes
	UINT8_T bDescriptorType;         // Descriptor Type (=4)
	UINT8_T bInterfaceNumber;        // Number of *this* Interface (0..)
	UINT8_T bAlternateSetting;       // Alternative for this Interface (if any)
	UINT8_T bNumEndpoints;           // No of EPs used by this IF (excl. EP0)
	UINT8_T bInterfaceClass;         // Interface Class Code
	UINT8_T bInterfaceSubClass;      // Interface Subclass Code
	UINT8_T bInterfaceProtocol;      // Interface Protocol Code
	UINT8_T iInterface;              // Index of String Desc for this Interface
} USBDEFS_INTERFACE_DESCRIPTOR_T;

#define USBDEFS_INTERFACE_DESC_SIZE	9

//----------------------------------------------------------------------------

// Standard Endpoint Descriptor
// according to USB1.1 spec page 203
//
typedef struct
{
	UINT8_T bLength;              // Size of this Descriptor in Bytes
	UINT8_T bDescriptorType;      // Descriptor Type (=5)
	UINT8_T bEndpointAddress;     // Endpoint Address (Number + Direction)
	UINT8_T bmAttributes;         // Endpoint Attributes (Transfer Type)
	IWORD_T wMaxPacketSize;       // Max. Endpoint Packet Size
	UINT8_T bInterval;            // Polling Interval (Interrupt) in ms
} USBDEFS_ENDPOINT_DESCRIPTOR_T;

#define USBDEFS_ENDPOINT_DESC_SIZE	7

//----------------------------------------------------------------------------

//
// String Descriptor
//
typedef struct
{
  UINT8_T   bLength;
  UINT8_T   bDescriptorType;
  UINT8_T   bString[1]; // variable size
} USBDEFS_STRING_DESCRIPTOR_T;

#define USBDEFS_STRING_DESC_SIZE	3

//----------------------------------------------------------------------------

//
// HID Descriptor
//
typedef struct
{
  UINT8_T   bLength;
  UINT8_T   bDescriptorType;
  IWORD_T   bcdHID; 
  UINT8_T   bCountryCode;
  UINT8_T   bNumDescriptors;
  UINT8_T   bDescriptorType1;
//  IWORD_T   wDescriptorLength1; 
  UINT8_T bDescriptorLength1Lo;
  UINT8_T bDescriptorLength1Hi;
} USBDEFS_HID_DESCRIPTOR_T;

#define USBDEFS_HID_DESC_SIZE	9



// # of active endpoints in configuration in addition to Control
// order of endpoint definition is determined by usbdefs_desc_struct[]
#define USBDEFS_NUM_ACTIVE_ENDPOINTS         1

// Configuration descriptor total length definition
#if (USBDEFS_NUM_ACTIVE_ENDPOINTS == 0)
#define USBDEFS_CONFIGURATION_DESCRIPTOR_TOTALLENGTH \
    (USBDEFS_CONFIGURATION_DESC_SIZE + \
	  USBDEFS_INTERFACE_DESC_SIZE + \
	  USBDEFS_HID_DESC_SIZE)
#elif (USBDEFS_NUM_ACTIVE_ENDPOINTS  == 1)
#define USBDEFS_CONFIGURATION_DESCRIPTOR_TOTALLENGTH \
    (USBDEFS_CONFIGURATION_DESC_SIZE + \
	  USBDEFS_INTERFACE_DESC_SIZE + \
	  USBDEFS_HID_DESC_SIZE +  \
	  USBDEFS_ENDPOINT_DESC_SIZE)
#elif (USBDEF_NUM_ACTIVE_ENDPOINTS == 2)
#define USBDEFS_CONFIGURATION_DESCRIPTOR_TOTALLENGTH \
    (USBDEFS_CONFIGURATION_DESC_SIZE + \
	  USBDEFS_INTERFACE_DESC_SIZE + \
	  USBDEFS_HID_DESC_SIZE +  \
	  USBDEFS_ENDPOINT_DESC_SIZE + \
	  USBDEFS_ENDPOINT_DESC_SIZE)
#else
#error Invalid setting for USBDEFS_NUM_ACTIVE_ENDPOINTS in USBDEFS.H
#endif


// Typedefs for USB Call-back functions 
typedef void (* USBDEFS_DEVICE_CFG_CB_FUNC_T)(INT16_T, UINT8_T, INT8_T, UINT16_T, UINT16_T);
typedef INT8_T (* USBDEFS_CLASS_VEND_REQ_CB_FUNC_T)(USBDEFS_SETUP_BUFFER_T *);
typedef void (* USBDEFS_BUS_STATE_CHG_CB_FUNC_T)(UINT8_T);
typedef void (* USBDEFS_SOF_DETECT_CB_FUNC_T)(UINT8_T, UINT8_T);
typedef void (* USBDEFS_FRAME_MATCH_CB_FUNC_T)(void);
typedef void (* USBDEFS_EP_TX_DONE_CB_FUNC_T)(UINT8_T, UINT8_T *, INT8_T);
typedef void (* USBDEFS_EP_RX_DATA_CB_FUNC_T)(UINT8_T, UINT8_T *, INT8_T);
typedef void (* USBDEFS_EP_HALT_CHG_CB_FUNC_T)(UINT8_T, UINT8_T);
typedef void * (* USBDEFS_GET_DESC_CB_FUNC_T)(UINT8_T, UINT8_T, UINT8_T, UINT8_T, UINT8_T);


// Definition for structure containing all call-back function addresses
typedef struct {
	USBDEFS_DEVICE_CFG_CB_FUNC_T device_config;
	USBDEFS_CLASS_VEND_REQ_CB_FUNC_T class_vend_req;
	USBDEFS_BUS_STATE_CHG_CB_FUNC_T bus_state_chg;
	USBDEFS_SOF_DETECT_CB_FUNC_T sof_detect;
	USBDEFS_FRAME_MATCH_CB_FUNC_T frame_match;
	USBDEFS_EP_TX_DONE_CB_FUNC_T ep_tx_done;
	USBDEFS_EP_RX_DATA_CB_FUNC_T ep_rx_data;
	USBDEFS_EP_HALT_CHG_CB_FUNC_T ep_halt_change;
	USBDEFS_GET_DESC_CB_FUNC_T get_desc;
} USBDEFS_CALLBACK_FUNCTIONS_T;

/* USB state definitions */
typedef enum { 
  USB_RESET_STATE = 0,
  USB_INIT_STATE,
  USB_CONFIG_STATE,
  USB_DISABLE_STATE
} USBDEFS_OPERATIONAL_STATES_T;

/* 
	USB Endpoint Operating State Information 
*/
typedef struct {
	INT8_T mode;
	INT8_T dir;
	UINT16_T packet_size;
} USBDEFS_EP_STATE_T;


/* 
	USB Configuration State Information 
*/
typedef struct {
	USBDEFS_OPERATIONAL_STATES_T usb_state;
	BOOLEAN_T usb_suspend;
	INT8_T num_interfaces;
	USBDEFS_EP_STATE_T ep_state[USBDEFS_NUM_ENDPOINTS];
} USBDEFS_CONFIG_STATE_T;


/* 
	Structure for Storing USB Tx and Rx Data 
*/
typedef struct
{
   UINT8_T * start;            // starting address for buffer
   UINT16_T current_ptr_pos;   // buffer offset in bytes for next data to tx or 
	                            // next location to store for rx
   UINT16_T length;            // length of the buffer in bytes
   UINT8_T flag;               // config/status flags
} USBDEFS_DATA_BUFFER_T;

/*
	USB Activity Counters
*/
typedef struct {
	UINT16_T ep_tx_packet_count[USBDEFS_NUM_ENDPOINTS];			// Counters
	UINT16_T ep_rx_packet_count[USBDEFS_NUM_ENDPOINTS];
	UINT16_T ep_tx_transfer_count[USBDEFS_NUM_ENDPOINTS];
	UINT16_T ep_rx_transfer_count[USBDEFS_NUM_ENDPOINTS];
	UINT16_T halt_count[USBDEFS_NUM_ENDPOINTS];
	UINT16_T unhalt_count[USBDEFS_NUM_ENDPOINTS];
	UINT16_T sof_count;
	UINT16_T asof_count;
	UINT16_T frm_mat_count;
	UINT16_T dev_cfg_count;
	UINT16_T vend_req_count;
	UINT16_T get_dev_desc_count;
	UINT16_T get_cfg_desc_count;
	UINT16_T wakeup_chg_count;
	UINT16_T resume_count;
	UINT16_T suspend_count;
	UINT16_T reset_count;
} USBDEFS_ACTIVITY_COUNTERS_T;




/*******************************************************************************
 *******************************************************************************
								Externals
********************************************************************************
*******************************************************************************/
extern const USBDEFS_DEVICE_DESCRIPTOR_T usbdefs_device_desc;
extern const USBDEFS_CONFIGURATION_DESCRIPTOR_T usbdefs_config_desc;
extern const USBDEFS_INTERFACE_DESCRIPTOR_T usbdefs_interface_desc;
extern const USBDEFS_ENDPOINT_DESCRIPTOR_T usbdefs_endpoint3_desc;
extern const USBDEFS_HID_DESCRIPTOR_T usbdefs_hid_desc;
extern const UINT8_T * const usbdefs_string_desc_table[];
extern const UINT8_T usbdefs_report_desc[];
extern const UINT8_T usbdefs_string0_desc[4];
extern UINT8_T usbdefs_string1_desc[USBDEFS_SD1LEN + 2];
extern UINT8_T usbdefs_string2_desc[USBDEFS_SD2LEN + 2];
extern UINT8_T usbdefs_string3_desc[USBDEFS_SD3LEN + 2];
extern UINT8_T usbdefs_string4_desc[USBDEFS_SD4LEN + 2];
extern UINT8_T usbdefs_string5_desc[USBDEFS_SD5LEN + 2];
extern UINT8_T usbdefs_string6_desc[USBDEFS_SD6LEN + 2];
extern const void * const usbdefs_desc_struct[9];

extern const UINT8_T usbdefs_hid_input_report_size[];

extern USBDEFS_SETUP_BUFFER_T usbdefs_setup_buffer;

extern USBDEFS_DATA_BUFFER_T usbdefs_ep_txbuf[USBDEFS_NUM_ENDPOINTS];
extern USBDEFS_DATA_BUFFER_T usbdefs_ep_rxbuf[USBDEFS_NUM_ENDPOINTS];

extern USBDEFS_CONFIG_STATE_T usbdefs_config_state;

extern USBDEFS_ACTIVITY_COUNTERS_T usbdefs_cnt;
	
extern UINT8_T usbdefs_rx_buffer[USBDEFS_NUM_ENDPOINTS][USBDEFS_RX_BUFFER_SIZE];

extern const UINT8_T usbdefs_fifo_size_table[USBDEFS_NUM_ENDPOINTS];
extern const UINT8_T usbdefs_pkt_size_table[4];

extern UINT8_T usbdefs_hid_report_id;
extern UINT8_T usbdefs_hid_report_type;

extern USBDEFS_CALLBACK_FUNCTIONS_T usbdefs_callback_func;


#endif /* _USBDEFS_H */

