/*******************************************************************************
********************************************************************************

   COPYRIGHT  2004, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   speccmds.h     Special command C Header File 

   About:         This module contains definitions for the Special command functions
                  for the AICONS01 project.

   Target CPU:    H8S/2215

   Written for:   Advanced Illumination, Inc.
                  24 Peavine Drive
                  Rochester, VT 05767
                  http://advancedillumination.com

   Contacts:      John Thrailkill
                  Phone:  (802) 767-3830
                  FAX:    (802) 767-3831
                  jthrailkill@advill.com

   Written by:    Rob Macklin
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 16 $

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/S1000/tar_src/Common/speccmds.h $
 * 
 * 16    12/30/05 3:29p Rmack
 * New function to get dynamic max_lv_volts value.
 * 
 * 15    8/19/05 10:14a Rmack
 * Added defines for access to new lighthead calculation and initial Sig1
 * id.
 * 
 * 14    2/17/05 3:40p Rmack
 * Added new special command identifier for handling timing bypass.
 * 
 * 13    2/17/05 10:52a Rmack
 * Added new command to get channel mode information.
 * 
 * 12    2/04/05 11:45a Rmack
 * Defined user data command and decode information.
 * 
 * 11    2/03/05 12:33p Rmack
 * Renamed current measurement command.
 * 
 * 10    2/02/05 3:58p Rmack
 * Added new command identifier
 * 
 * 9     10/29/04 8:06a Rmack
 * Defined new IDs for Signatech I detection function.
 * 
 * 8     9/30/04 11:50a Rmack
 * Added function for lighthead serial number
 * 
 * 7     8/24/04 8:03a Rmack
 * DTC based triggering of all four channels with  <  2usec delay.  Basic
 * TK_PULSE command interface implemented.
 * 
 * 6     8/11/04 4:40p Jwayg
 * Final check-in under JWayg name, code currently setup to run LED tests.
 * 
 * 5     7/01/04 10:30a Jwayg
 * Last code release before transfer of source files to new programmer.
 * 
 * 4     6/07/04 9:54a Jwayg
 * S1000 - -005 firmware level - pulses created programatically via
 * special commands
 * 
 * 3     6/01/04 5:04p Jwayg
 * S1000 - in process version with test pulses - being geared for Jon's
 * hardware testing - 8200-000068-003 level
 * 
 * 2     5/17/04 10:57a Jwayg
 * Common Files for Initial Running Version of S1000 
 * 
 * 1     3/30/04 3:31p Rmack
 *
********************************************************************************
*******************************************************************************/
#ifndef _SPECCMDS_H
#define _SPECCMDS_H

/* Define special functions */
#define SPECCMD_STANDBY_CMD               0xFF
#define SPECCMD_RESET_COUNTERS_CMD        0x0100
#define SPECCMD_RESET_COUNTER_INDEX_CMD   0x0101
#define SPECCMD_GET_ROTARY_SW_CMD         0x0102
#define SPECCMD_CLEAR_ERRORS_CMD          0x0300
#define SPECCMD_SET_UNIT_ID_CMD           0x0400
#define SPECCMD_INVOKE_LOADER_CMD         0x0800
#define SPECCMD_RESTART_CMD               0x0801
#define SPECCMD_FLASH_INIT_CMD            0x0810
#define SPECCMD_FLASH_ERASE_CMD           0x0811
#define SPECCMD_FLASH_PROG_CMD            0x0812
#define SPECCMD_FLASH_SHUTDOWN_CMD        0x0813

/* Added by RM, 7/7/04 for AI lighthead testing */
#define SPECCMD_TRIGGER_ALL_CMD           0x0902
#define SPECCMD_SET_PULSE_WIDTH_CMD       0x0910
#define SPECCMD_GET_PULSE_WIDTH_CMD       0x0911
#define SPECCMD_SET_PULSE_CURRENT_CMD     0x0912
#define SPECCMD_GET_PULSE_CURRENT_CMD     0x0913
#define SPECCMD_SET_HV_SUPPLY_CMD         0x0914
#define SPECCMD_GET_HV_SUPPLY_CMD         0x0915
#define SPECCMD_SET_INT_TRIG_CMD          0x0916
#define SPECCMD_GET_INT_TRIG_CMD          0x0917
#define SPECCMD_GET_LH_SERIALNUM_CMD      0x0051
#define SPECCMD_GET_SIG1_ID_CMD           0x0052

#define SPECCMD_MEASURE_DRIVE_CURRENT_CMD 0x0060
#define SPECCMD_CALC_CURR_AT_MAX_VOLTS_CMD 0x061
#define SPECCMD_GET_ACTIVE_SIG1_CODE_CMD  0x0062
#define SPECCMD_GET_MAX_LV_VOLTS_CMD      0x0063

#define SPECCMD_GET_USER_DATA_CMD         0x0070
#define SPECCMD_GET_CHAN_MODE_CMD         0x0071
#define SPECCMD_SET_TIMING_BYPASS_CMD     0x0072
#define SPECCMD_SET_INTERNAL_TRIG_CMD     0x0073


/* Define masks used in User Data command */
#define SPECCMD_ROTARY_SWITCH_MASK        0x00000FFF
#define SPECCMD_DIP_SWITCH_MASK           0x0000F000
#define SPECCMD_TRIGGER_MASK              0x000F0000
#define SPECCMD_CONNECTED_MASK            0x00F00000
#define SPECCMD_SUPPLY_MASK               0x0F000000
#define SPECCMD_EXPANSION_MASK            0xF0000000

/* Define shifts used to justify user data */
#define SPECCMD_ROTARY_SWITCH_SHIFT       0
#define SPECCMD_DIP_SWITCH_SHIFT          12
#define SPECCMD_TRIGGER_SHIFT             16
#define SPECCMD_CONNECTED_SHIFT           20
#define SPECCMD_SUPPLY_SHIFT              24
#define SPECCMD_EXPANSION_SHIFT           28




#endif /* _SPECCMDS_H */
