Imports VB = Microsoft.VisualBasic

Public Structure UserDataStruct
    Public Rotary As Short
    Public Dip As Byte
    Public Trigger As Byte
    Public Connect As Byte
    Public Supply As Byte
    Public Expansion As Byte
    Public Shared Operator <>(ByVal ud1 As UserDataStruct, _
               ByVal ud2 As UserDataStruct) As Boolean
        Return (Not (ud1.Connect = ud2.Connect))
    End Operator
    Public Shared Operator =(ByVal ud1 As UserDataStruct, _
               ByVal ud2 As UserDataStruct) As Boolean
        Return ((ud1.Connect = ud2.Connect) And _
                 (ud1.Dip = ud2.Dip) And _
                 (ud1.Expansion = ud2.Expansion) And _
                 (ud1.Rotary = ud2.Rotary) And _
                 (ud1.Supply = ud2.Supply) And _
                 (ud1.Trigger = ud2.Trigger))
    End Operator
End Structure

'The api does not specifically enumerate Gated DC
'We do in the app because of the user interface
Public Enum APP_MODE_E
    APP_MODE_PULSED = 0
    APP_MODE_CONTINUOUS
    APP_MODE_TIMING_BYPASS
    APP_MODE_INTERNAL_TRIGGER
    APP_MODE_GATED_CONTINUOUS
End Enum

Public Class CPulsar
    Private Const TKPULSE_NUM_CHAN As Short = 1

    Const TKCMDSET_H8_PRODUCT_NAME As String = "Ai Pulsar 320 Controller"
    Const E_SUCCESS As Short = 0

    Private Declare Sub Sleep Lib "kernel32.dll" (ByVal dwMilliseconds As Long)

    'Port is a USB item and is an argument to most dll usb calls
    Public Port As Byte
    Public Name As String

    'The Pulsar HW and FW versions come out as arrays of bytes
    Dim DllVersionNumber As Short
    'Dim PulsarFWVersion(256) As Byte
    'Dim PulsarHWVersion(256) As Byte
    Dim PulsarFWVersion As String = ""
    Dim PulsarHWVersion As String = ""
    Dim PulsarSerialNumber As String = ""

    Dim UserData As UserDataStruct

    Public PulsarConnected As Boolean 'True means the Pulsar is connected

    Dim ChannelConfig(4) As API_CHAN_CONFIG_STRUCT_T
    Dim ChannelConfigWriteSaveType As Byte = API_CHAN_CONFIG_DEFAULT
    Dim ChannelConfigReadSaveType As Byte = API_CHAN_CONFIG_DEFAULT

    ' Variables indicating jumper installation
    ' 0 Jumper not installed
    ' 1 Jumper installed
    Dim HVJumpers(4) As Boolean
    Dim LVJumpers(4) As Boolean

    Public Event ConnectDone(ByVal connected As Boolean)

    Public Function Initialize() As Short
        Dim I As Short
        Dim Retval As Short

        Retval = 0
        'Try to get the dll version number. 
        'Terminate in a fairly orderly fashion on a DllNotFoundException.
        'If we get past this, we assume that the dll is always available
        'for further calls. So Try-Catch will not be used for any other
        'dll calls.
        Try
            DllVersionNumber = api_get_dll_version()
        Catch ex As System.DllNotFoundException
            MsgBox("A supporting file is missing. Application will exit. " + Chr(13) + Chr(10) + ex.ToString, MsgBoxStyle.Critical)
            Return 1
        Finally
        End Try
        'Port = 0
        PulsarConnected = False
        For I = 0 To TKPULSE_NUM_CHAN
            ChannelConfig(I).current = 0.0
            ChannelConfig(I).width = 0
            ChannelConfig(I).Delay = 0
        Next
        Return Retval
    End Function
    '*****************************************************
    'Connect

    'Attempts a USB Connection to the Pulsar
    'Gets FW and HW versions if a connection is made

    'Parameters: 
    'Inputs: ShowResult - Show connect result even if it has not changed
    'Returns: None
    'Globals: PulsarConnected
    '         PulsarFWVersion
    '         PulsarHWVersion
    '*****************************************************
    Public Function Connect(ByVal Parent As Object) As Boolean
        Dim e_code As Short
        Dim e_text As String = ""
        Dim temp_string_1 As String = ""
        Dim temp_string_2 As String = ""
        Dim temp_string_3 As String = ""
        Static Dim Iteration As Byte = 0
        Static Dim CmdFailCount As Byte = 0
        If Not PulsarConnected Then
            e_code = api_init(Port, TKCMDSET_H8_PRODUCT_NAME, "")
            If (e_code <> E_SUCCESS) Then
                api_get_error_text(e_code, e_text, 256)
                PulsarConnected = False
            Else
                PulsarConnected = True
                'RaiseEvent ConnectDone(PulsarConnected)
                Parent.Invoke(Parent.HandleConnectDelegate)
            End If
        End If

        If PulsarConnected Then
            'If we are here the init command succeeded, so there is a pulsar.
            'Now assume all of the commands that depend on a pulsar will succeed
            'for the remainder of the sub.
            Select Case Iteration
                Case 1 'get pulsar version
                    'Need to give these strings an initial length
                    temp_string_1 = LSet(temp_string_1, 256)
                    temp_string_2 = LSet(temp_string_2, 256)
                    temp_string_3 = LSet(temp_string_3, 256)
                    e_code = api_get_pulsar_version(Port, _
                                                            temp_string_1, _
                                                            temp_string_2, _
                                                            temp_string_3)
                    If e_code <> 0 Then
                        CmdFailCount += 1
                    Else
                        CmdFailCount = 0
                        Iteration += 1
                        PulsarFWVersion = CTrim(temp_string_1)
                        PulsarHWVersion = CTrim(temp_string_2)
                        PulsarSerialNumber = CTrim(temp_string_3)
                    End If
                Case 2
                    e_code = api_get_user_data(Port, UserData.Rotary, _
                                                     UserData.Dip, _
                                                     UserData.Trigger, _
                                                     UserData.Connect, _
                                                     UserData.Supply, _
                                                     UserData.Expansion)
                    If e_code <> 0 Then
                        CmdFailCount += 1
                    Else
                        CmdFailCount = 0
                        Iteration = 0
                        If (UserData.Supply And API_SUPPLY_VIN_ON) <> 0 Then
                            ChannelConfigWriteSaveType = API_CHAN_CONFIG_ACTIVE Or API_CHAN_CONFIG_DEFAULT
                            'ChannelConfigWriteSaveType = API_CHAN_CONFIG_ACTIVE
                            ChannelConfigReadSaveType = API_CHAN_CONFIG_ACTIVE
                        Else
                            ChannelConfigWriteSaveType = API_CHAN_CONFIG_DEFAULT
                            ChannelConfigReadSaveType = API_CHAN_CONFIG_DEFAULT
                        End If
                    End If
            End Select

            If CmdFailCount > 3 Then
                Parent.Invoke(Parent.HandleConnectDelegate)
            End If
        End If
        Return PulsarConnected
    End Function

    'Warning a big pw could be -1
    Public Function ProtectChannelTiming(ByVal channel As Byte, ByRef flags As Byte) As Short
        Dim ch_mask As Byte
        Dim max_pw As Integer
        Dim e_code As Short

        flags = 0
        ch_mask = 1 << channel
        e_code = api_calc_max_width(Port, channel + 1, ChannelConfig(channel).current, max_pw)
        If e_code = E_SUCCESS Then
            If max_pw < ChannelConfig(channel).width And max_pw > 0 Then
                ChannelConfig(channel).width = max_pw
                flags = ch_mask
            End If
        End If
        Return e_code
    End Function

    ' Subtracts .01 from any float that is not a whole number
    Function FudgeFloat(ByVal f As Single) As Single
        If Fix(f) = f Then
            Return f
        End If
        Return f - 0.01
    End Function

    Public Function ProtectChannelCurrent(ByVal Channel As Byte, _
                                          ByRef flags As Byte) As Short
        Dim ch_mask As Byte
        Dim max_i As Single
        Dim e_code As Short

        flags = 0
        ch_mask = 1 << Channel
        e_code = api_calc_max_current(Port, Channel + 1, ChannelConfig(Channel).width, max_i)
        If e_code = E_SUCCESS Then
            max_i = FudgeFloat(max_i)
            If max_i < ChannelConfig(Channel).current Then
                ChannelConfig(Channel).current = max_i
                flags = ch_mask
            End If
        End If
        Return e_code
    End Function

    Public Function ProtectChannel(ByVal Channel As Byte, ByVal AdjustTiming As Boolean, ByRef flags As Byte) As Short
        Dim e_code As Short
        Dim temp As Short

        If AdjustTiming And ChannelConfig(Channel).mode <> API_MODE_E.API_MODE_CONTINUOUS Then
            ' check for absolute max current (max current at 0 PW)
            temp = ChannelConfig(Channel).width
            ChannelConfig(Channel).width = 1
            e_code = ProtectChannelCurrent(Channel, flags)
            ChannelConfig(Channel).width = temp
            e_code = ProtectChannelTiming(Channel, flags)
        Else
            e_code = ProtectChannelCurrent(Channel, flags)
        End If

        Return e_code
    End Function

    ' mode is in APP_MODE_E format so it must be converted
    ' to API_MODE_E format befor going to the pulser
    Public Function SetModeDirect(ByVal channel As Byte, _
                            ByVal mode As Byte, _
                            ByVal equalize As Boolean) As Short
        Dim e_code As Short
        Dim send_mode As Byte
        Dim i As Byte

        ' If continuous mode make sure no trigger
        If mode = APP_MODE_E.APP_MODE_CONTINUOUS Or _
            mode = APP_MODE_E.APP_MODE_GATED_CONTINUOUS Then
            ChannelConfig(channel).trigger = API_TRIG_E.API_TRIG_NONE
        End If
        ' If gated continuous mode then just make it continuous
        If mode = APP_MODE_E.APP_MODE_GATED_CONTINUOUS Then
            send_mode = API_MODE_E.API_MODE_CONTINUOUS
            ChannelConfig(channel).reserved_byte = 1
        Else
            send_mode = mode
        End If
        ' now if we are going from pulsed to continuous then set all currents to 0
        If send_mode = API_MODE_E.API_MODE_CONTINUOUS And _
           ChannelConfig(channel).mode <> API_MODE_E.API_MODE_CONTINUOUS Then
            ChannelConfig(channel).current = 0
            equalize = True
        End If

        For i = 0 To TKPULSE_NUM_CHAN
            If equalize = True Then
                ChannelConfig(i) = ChannelConfig(channel)
            End If
            ' for now all channels get the same mode
            ChannelConfig(i).mode = send_mode
            e_code = api_set_channel_config(Port, _
                      i + 1, _
                      ChannelConfigWriteSaveType, _
                      ChannelConfig(i))
            If e_code <> 0 Then Return e_code
        Next
        Return e_code
    End Function

    Public Sub SetCurrent(ByVal channel As Byte, _
                          ByVal new_current As Single, _
                          ByVal synch As Boolean, _
                          ByVal equalize As Boolean)

        Dim d_current As Single
        Dim i As Byte

        If equalize = True Then
            For i = 0 To TKPULSE_NUM_CHAN
                ChannelConfig(i).current = new_current
            Next
        ElseIf synch = True Then
            d_current = new_current - ChannelConfig(channel).current
            For i = 0 To TKPULSE_NUM_CHAN
                ChannelConfig(i).current = ChannelConfig(i).current + d_current
            Next
        Else
            ChannelConfig(channel).current = new_current
        End If
    End Sub

    Public Sub SetPW(ByVal channel As Byte, _
                     ByVal new_pw As Integer, _
                     ByVal synch As Boolean, _
                     ByVal equalize As Boolean)

        Dim d_pw As Single
        Dim i As Byte

        If equalize = True Then
            For i = 0 To TKPULSE_NUM_CHAN
                ChannelConfig(i).width = new_pw
            Next
        ElseIf synch = True Then
            d_pw = new_pw - ChannelConfig(channel).width
            For i = 0 To TKPULSE_NUM_CHAN
                ChannelConfig(i).width = ChannelConfig(i).width + d_pw
            Next
        Else
            ChannelConfig(channel).width = new_pw
        End If
    End Sub

    Public Sub SetDelay(ByVal channel As Byte, _
                        ByVal new_delay As Short, _
                        ByVal equalize As Boolean)

        Dim i As Byte

        If equalize = True Then
            For i = 0 To TKPULSE_NUM_CHAN
                ChannelConfig(i).Delay = new_delay
            Next
        Else
            ChannelConfig(channel).Delay = new_delay
        End If
    End Sub

    Public Sub SetTrigger(ByVal channel As Byte, _
                          ByVal new_trig_src As Byte, _
                          ByVal new_trig_rising As Byte, _
                          ByVal equalize As Boolean)

        Dim i As Byte

        If equalize = True Then
            For i = 0 To TKPULSE_NUM_CHAN
                ChannelConfig(i).trigger = new_trig_src
                ChannelConfig(i).rising = new_trig_rising
            Next
        Else
            ChannelConfig(channel).trigger = new_trig_src
            ChannelConfig(channel).rising = new_trig_rising
        End If
    End Sub

    'SafeSetMode
    ' Sets the mode bits in the config structure for channel
    ' Checks for legal mode configuration based on the jumpers
    ' Here is the rule for legality:
    '   IF A CHANNEL HAS THE HV JUMPER INSTALLED, THIS CHANNEL
    '   CANNOT BE IN A CONTINUOUS MODE WHILE ANY OTHER CHANNEL
    '   IS IN A PULSED MODE. THIS WILL CAUSE THE CHANNEL TO 
    '   DRAW CONTINUOUS CURRENT FROM THE HV SUPPLY WHICH WILL
    '   BURN UP THE CONTROLLER.
    '
    ' We obey the rule as follows:
    '   -If the new_mode is a continuous mode:
    '       -If this channel has no LV jumper an error is returned
    '       -If this channel has a HV jumper all other channels are
    '        switched to the same continuous mode
    '   -If the new_mode is a pulsed mode:
    '       -If this channel has no HV jumper an error is returned
    '       -All channels with a HV jumper are switched to pulsed mode
    Public Function SafeSetMode(ByVal channel As Byte, _
                                ByVal new_mode As Byte, _
                                ByVal equalize As Boolean) As Short

    End Function


    'Sends the current channel settings to the pulsar
    'channel 0-3 channel  >8 all channels
    'adjust_timing 1 adjust timing 0 adjust_current
    'prot_flags bitwise indication of channels that had protection applied
    'force_all true means update all channels
    Public Function UpdateChannel(ByVal channel As Byte, _
                                  ByVal adjust_timing As Boolean, _
                                  ByRef prot_flags As Byte, _
                                  ByVal force_all As Boolean) As Short
        Dim e_code As Short
        Dim i As Byte
        Dim temp_flags As Byte

        e_code = E_SUCCESS
        If channel >= 8 Or force_all = True Then
            ' Protect all channels
            temp_flags = 0
            For i = 0 To TKPULSE_NUM_CHAN
                e_code = ProtectChannel(i, adjust_timing, prot_flags)
                If e_code <> E_SUCCESS Then
                    Return e_code
                End If
                temp_flags = temp_flags Or prot_flags
            Next
            prot_flags = temp_flags
        Else
            ' Only protect the channel in question
            e_code = ProtectChannel(channel, adjust_timing, prot_flags)
            If e_code <> E_SUCCESS Then
                Return e_code
            End If
        End If
        'do all channels here if synched or equalize
        If channel >= 8 Or force_all = True Then
            For i = 0 To TKPULSE_NUM_CHAN
                'Sleep(200)
                e_code = api_set_channel_config(Port, _
                          i + 1, _
                          ChannelConfigWriteSaveType, _
                          ChannelConfig(i))
                If e_code <> E_SUCCESS Then
                    Return e_code
                End If
            Next
        Else
            e_code = api_set_channel_config(Port, _
                      channel + 1, _
                      ChannelConfigWriteSaveType, _
                      ChannelConfig(channel))
            If e_code <> E_SUCCESS Then
                Return e_code
            End If
        End If
        Return e_code
    End Function

    Public Sub RefreshChannel(ByVal channel As Byte)
        Dim i As Byte
        Dim e_code As Short

        '        Dim cc As API_CHAN_CONFIG_STRUCT_T

        If channel < 4 Then
            e_code = api_get_channel_config(Port, _
                      channel + 1, _
                      ChannelConfigReadSaveType, _
                      ChannelConfig(channel))
        Else
            For i = 0 To TKPULSE_NUM_CHAN
                e_code = api_get_channel_config(Port, _
                          i + 1, _
                          ChannelConfigReadSaveType, _
                          ChannelConfig(i))
            Next
        End If
    End Sub

    Public Function GetStoredData(ByRef ud As Byte, ByVal len As Byte) As Short
        Dim e_code As Short

        e_code = api_get_stored_data(Port, ud, len)
        Return e_code
    End Function

    Public Function SaveStoredData(ByRef ud As Byte, ByVal len As Byte) As Short
        Dim e_code As Short

        e_code = api_set_stored_data(Port, ud, len)
        Return e_code
    End Function

    Public Function TriggerTest(ByVal trigger As Byte) As Short
        Dim e_code As Short
        e_code = api_trigger(Port, trigger)
        Return e_code
    End Function
    Public Sub GetConnectStatusText(ByRef cst As String)
        cst = "DLL Version " + Str(DllVersionNumber / 256) _
                             + "." _
                             + Str(DllVersionNumber And &HFF) _
                             + Chr(13) + Chr(10)
        If Not PulsarConnected Then
            cst = cst + "Controller Not Connected"
        Else
            cst = cst + "FW: " + PulsarFWVersion + Chr(13) + Chr(10)
            cst = cst + "HW: " + PulsarHWVersion + Chr(13) + Chr(10)
        End If
    End Sub

    Public Function ShowError(ByVal e_code As Short, ByRef err_str As String) As Boolean
        Dim name_vect As String = ""
        Dim text_vect As String = ""

        name_vect = LSet(name_vect, 256)
        text_vect = LSet(text_vect, 256)
        If e_code = E_SUCCESS Then
            err_str = "Ok"
            Return False
        End If
        api_get_error_name(e_code, name_vect, 256)
        name_vect = CTrim(name_vect)
        api_get_error_text(e_code, text_vect, 256)
        text_vect = CTrim(text_vect)
        err_str = "Error: " + Str(e_code) + Chr(10) + Chr(13)
        err_str = err_str + name_vect
        err_str = err_str + Chr(10) + Chr(13)
        err_str = err_str + text_vect
    End Function


    Public Sub Dump()
        Dim m As String
        Dim i As Short

        If PulsarConnected = True Then
            m = "Connected" + Chr(13) + Chr(10)
        Else
            m = "Not Connected" + Chr(13) + Chr(10)
        End If
        For i = 0 To TKPULSE_NUM_CHAN
            m = m + Format(ChannelConfig(i).current, "##0.000") + " : "
            m = m + Format(ChannelConfig(i).width, "###0") + " : "
            m = m + Format(ChannelConfig(i).Delay, "###0") + Chr(13) + Chr(10)
        Next
        m = m + Chr(13) + Chr(10) + Chr(13) + Chr(10)
        For i = 0 To TKPULSE_NUM_CHAN
            m = m + Format(ChannelConfig(i).reserved_byte, "###0") + " : "
            m = m + Format(ChannelConfig(i).trigger, "###0") + Chr(13) + Chr(10)
        Next
        MsgBox(m)
    End Sub

    Public Function GetPulseWidth(ByVal channel As Byte) As Integer
        Return ChannelConfig(channel).width
    End Function

    Public Function GetDelay(ByVal channel As Byte) As Short
        Return ChannelConfig(channel).Delay
    End Function

    Public Function GetCurrent(ByVal channel As Byte) As Single
        Return ChannelConfig(channel).current
    End Function

    Public Function GetTriggerSource(ByVal channel As Byte) As Byte
        Return ChannelConfig(channel).trigger
    End Function

    Public Function GetTriggerSourceRising(ByVal channel As Byte) As Byte
        Return ChannelConfig(channel).rising
    End Function

    Public Function GetMode(ByVal channel As Byte) As Byte
        Return ChannelConfig(channel).mode
    End Function

    Public Function GetSupplyStatus() As Byte
        Return UserData.Supply
    End Function

    Public Function GetConnectStatus() As Byte
        Return UserData.Connect
    End Function

    Public Function IsMainPowerOn() As Boolean
        Dim temp_byte As Byte

        temp_byte = UserData.Supply And API_SUPPLY_VIN_ON
        If temp_byte <> 0 Then
            Return True
        End If
        Return False
    End Function

    Public Function IsLHConnected() As Boolean
        Dim temp_byte As Byte

        temp_byte = UserData.Connect And API_CONN_LH_DETECT
        If temp_byte <> 0 Then
            Return True
        End If
        Return False
    End Function
End Class
