﻿Imports System.Threading
Imports System.Windows.Forms

Public Class Config
    Const NUM_CHANNELS As Byte = 2
    Private WithEvents myPulsar As CPulsar2
    Public Event evConnect()
    Private bRefreshSettings As Boolean
    Public Declare Function GetSysColor _
        Lib "user32" Alias "GetSysColor" _
        (ByVal nIndex As Long) As Long
    Private Sub OK_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        Me.DialogResult = System.Windows.Forms.DialogResult.OK
        Me.Close()
    End Sub
    Private Sub Cancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
        Me.Close()
    End Sub

    Private Sub Config_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        myPulsar.StopPulsar()
        EnableControls(False)
    End Sub

    Private Sub Config_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.GotFocus
        EnableControls(myPulsar.PulsarConnected)
    End Sub

    Private Sub Config_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Disable all controls until a connection is established
        EnableControls(False)
    End Sub
    Private Sub EnableControls(ByVal EnabledFlag As Boolean)
        '        Enabled = True
        Output1Group.Enabled = EnabledFlag
        Output2Group.Enabled = EnabledFlag
        SynchronizeGroup.Enabled = EnabledFlag
        ProtectionGroup.Enabled = EnabledFlag
        ProgramButton.Enabled = EnabledFlag
        RefreshControls.Enabled = EnabledFlag
        CType(MdiParent, Pulsar320AppMain).ChildConnected(EnabledFlag)
        If EnabledFlag Then
            'Me.BackColor = Color.FromKnownColor(KnownColor.Control)
            'ChannelConfig1.BackColor = Color.FromKnownColor(KnownColor.Control)
            'ChannelConfig2.BackColor = Color.FromKnownColor(KnownColor.Control)
        Else
            'Me.BackColor = Color.Red
            'ChannelConfig1.BackColor = Color.Red
            'ChannelConfig2.BackColor = Color.Red
        End If
    End Sub
    Public Sub New(ByVal port As Byte, ByVal name As String, ByRef parentApp As Pulsar320AppMain)

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call
        MdiParent = parentApp

        ' Add any initialization after the InitializeComponent() call.
        myPulsar = New CPulsar2
        myPulsar.Initialize(Me, port, name)

    End Sub
    Public Delegate Sub CheckDelegate()
    Public CheckSettingsDelegate As CheckDelegate = AddressOf CheckSettings
    Public Sub CheckSettings()
        Dim dialogSettings(1) As API_CHAN_CONFIG_STRUCT_T
        Dim deviceSettings(1) As API_CHAN_CONFIG_STRUCT_T
        Dim lightheadConnect As Short
        Dim settingsMatch As Boolean
        Dim byChan As Byte

        If (bRefreshSettings) Then
            bRefreshSettings = False
            Refresh_Click(Me, Nothing)
        End If
        settingsMatch = True
        'Get dialog and pulsar settings
        ChannelConfig1.GetSettings(dialogSettings(0))
        ChannelConfig2.GetSettings(dialogSettings(1))
        myPulsar.GetSettings(deviceSettings(0), 1)
        myPulsar.GetSettings(deviceSettings(1), 2)
        lightheadConnect = myPulsar.GetLightheadStatus()

        For byChan = 1 To NUM_CHANNELS
            'Check channel 2, first check if connected
            If ((lightheadConnect And (1 << (byChan - 1))) = 0) Then
                'Not connected, disable controls
                GetChanConfig(byChan).Enabled = False
                GetChanConfig(byChan).BackColor = Color.DarkGray
            Else
                'if connected, then compare settings
                If (Not GetChanConfig(byChan).Enabled) Then
                    'The first time channel connected, refresh dialog settings
                    GetChanConfig(byChan).Enabled = True
                    GetChanConfig(byChan).BackColor = Color.FromKnownColor(KnownColor.Control)
                    RefreshSettings(byChan)
                End If
                If (dialogSettings(byChan - 1) <> deviceSettings(byChan - 1)) Then
                    settingsMatch = False
                End If

            End If
        Next

        If Not settingsMatch Then
            ProgramLabel.BackColor = Color.Red
        Else
            ProgramLabel.BackColor = Color.FromKnownColor(KnownColor.Control)
        End If
    End Sub
    Public Delegate Sub ConnectDelegate()
    Public HandleConnectDelegate As ConnectDelegate = AddressOf HandleConnect
    Public Sub HandleConnect()
        Dim devSettings As API_CHAN_CONFIG_STRUCT_T
        EnableControls(myPulsar.PulsarConnected)
        myPulsar.GetSettings(devSettings, 1)
        ChannelConfig1.SetSettings(devSettings)
        ChannelConfig1.AdjustScales()
        myPulsar.GetSettings(devSettings, 2)
        ChannelConfig2.SetSettings(devSettings)
        ChannelConfig2.AdjustScales()
    End Sub
    Public Sub ChannelParamsChanged(ByVal channel As ChannelConfig) Handles ChannelConfig1.evSettingsChanged, ChannelConfig2.evSettingsChanged
        Dim dialogSettings(2) As API_CHAN_CONFIG_STRUCT_T
        Dim deviceSettings(2) As API_CHAN_CONFIG_STRUCT_T
        Dim chanScales(2) As ChannelConfig.SCALE_VALUES
        Dim srcChan As Integer
        Dim dstchan As Integer
        Dim maxFreq As Short
        If channel.Name = ChannelConfig1.Name Then
            srcChan = 0
            dstchan = 1
        Else
            srcChan = 1
            dstchan = 0
        End If

        'Check for special setup mode
        If GetChanConfig(srcChan + 1).ModeSetup.Checked Then
            'Send settings immediately
            myPulsar.ProgramSetupMode(srcChan + 1)
        Else




            ChannelConfig1.GetSettings(dialogSettings(0))
            ChannelConfig1.GetScales(chanScales(0))
            ChannelConfig2.GetSettings(dialogSettings(1))
            ChannelConfig2.GetScales(chanScales(1))

            ' Update max frequency
            myPulsar.GetMaxFreq(srcChan + 1, dialogSettings(srcChan).width, dialogSettings(srcChan).current, maxFreq)
            If channel.Name = ChannelConfig1.Name Then
                ChannelConfig1.MaxFreqBox.Text = maxFreq.ToString
            Else
                ChannelConfig2.MaxFreqBox.Text = maxFreq.ToString
            End If


            If Synchronize.Checked Then
                dialogSettings(dstchan).width = dialogSettings(srcChan).width
                dialogSettings(dstchan).Delay = dialogSettings(srcChan).Delay
                dialogSettings(dstchan).offtime = dialogSettings(srcChan).offtime
                dialogSettings(dstchan).repeat_cnt = dialogSettings(srcChan).repeat_cnt
                dialogSettings(dstchan).Delay = dialogSettings(srcChan).Delay
                chanScales(dstchan).pw = chanScales(srcChan).pw
                chanScales(dstchan).delay = chanScales(srcChan).delay
            End If
            If Equalize.Checked Then
                dialogSettings(dstchan).current = dialogSettings(srcChan).current
                chanScales(dstchan).current = chanScales(srcChan).current
            End If
            'if sync or equalized, match triggers and mode
            If Equalize.Checked Or Synchronize.Checked Then
                dialogSettings(dstchan).mode = dialogSettings(srcChan).mode
                dialogSettings(dstchan).trigger = dialogSettings(srcChan).trigger
                dialogSettings(dstchan).rising = dialogSettings(srcChan).rising
            End If
            GetChanConfig(dstchan + 1).SetSettings(dialogSettings(dstchan))
            GetChanConfig(dstchan + 1).SetScales(chanScales(dstchan))

            '        ChannelConfig1.SetMaxCurrent(myPulsar.GetMaxCurrent(1, chanSettings(0).width))
        End If
    End Sub
    Private Function GetChanConfig(ByVal index As Integer) As ChannelConfig
        If index = 2 Then
            Return ChannelConfig2
        Else
            Return ChannelConfig1
        End If
    End Function

    Private Sub ProgramButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProgramButton.Click
        Dim settings As API_CHAN_CONFIG_STRUCT_T
        EnableControls(False) ' = False 'Controls(False)
        'CType(MdiParent, Pulsar320AppMain).ToolStripStatusLabel.Text = "Programming..."
        ChannelConfig1.LightheadData.Text = ""
        ChannelConfig2.LightheadData.Text = ""
        ChannelConfig1.GetSettings(settings)
        myPulsar.ProgramDevice(settings, 1, AdjustCurrent.Checked)
        ChannelConfig2.GetSettings(settings)
        myPulsar.ProgramDevice(settings, 2, AdjustCurrent.Checked)
        EnableControls(True) 'Enabled = True 'Controls(True)
        Me.Activate()
        bRefreshSettings = True
        'CType(MdiParent, Pulsar320AppMain).ToolStripStatusLabel.Text = "Programming...Complete"
    End Sub
    Private Sub Config_LostFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LostFocus
        EnableControls(myPulsar.PulsarConnected)
    End Sub
    Public Function GetPulsar() As CPulsar2
        Return myPulsar
    End Function
    Private Sub RefreshSettings(ByVal p_Chan As Short)
        Dim chanSettings As API_CHAN_CONFIG_STRUCT_T
        Dim maxFreq As Short
        myPulsar.GetSettings(chanSettings, p_Chan)
        GetChanConfig(p_Chan).SetSettings(chanSettings)
        myPulsar.GetMaxFreq(p_Chan, chanSettings.width, chanSettings.current, maxFreq)
        If p_Chan = 1 Then
            ChannelConfig1.MaxFreqBox.Text = maxFreq.ToString
        Else
            ChannelConfig2.MaxFreqBox.Text = maxFreq.ToString
        End If
    End Sub

    Private Sub trigger(ByRef p_chan As ChannelConfig, ByVal trigger As Byte) Handles ChannelConfig1.evTrigger, ChannelConfig2.evTrigger
        If p_chan.Name = ChannelConfig1.Name Then
            myPulsar.Trigger(0, trigger)
        Else
            myPulsar.Trigger(1, trigger)
        End If
    End Sub

    Private Sub Refresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RefreshControls.Click
        RefreshSettings(1)
        RefreshSettings(2)
    End Sub

    Private Sub handleError(ByVal p_chan As Byte, ByVal p_ecode As Short, ByVal p_errName As String, ByVal p_errDesc As String) Handles myPulsar.evError
        Dim msg As String
        msg = "Error" & vbCrLf & p_ecode
        msg = "Error" & vbCrLf & p_ecode & " - " & p_errName & p_errDesc
        GetChanConfig(p_chan).LightheadData.Text = "Error" & Environment.NewLine & _
                                                   p_ecode & "-" & p_errName & Environment.NewLine & p_errDesc
        'msg = p_errDesc
        'msg = "Error " & p_ecode & ":" & p_errDesc + ":" + p_errName

        'CType(MdiParent, Pulsar320AppMain).ToolStripStatusLabel.Text = "Error " & p_ecode & _
        '                                                                       ":" & p_errName & ":" & p_errDesc

    End Sub
    Friend Function GetDialogSettings() As PulsarData
        Dim dialogSettings As New PulsarData
        ChannelConfig1.GetSettings(dialogSettings.Settings(0))
        ChannelConfig2.GetSettings(dialogSettings.Settings(1))
        Return dialogSettings
    End Function
    Friend Sub SetDialogSettings(ByVal dialogSettings As PulsarData)
        ChannelConfig1.SetSettings(dialogSettings.Settings(0))
        ChannelConfig2.SetSettings(dialogSettings.Settings(1))
    End Sub
End Class
